/*
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import {Layout, WordDefinition} from '../domain/layout';

const esIsch: WordDefinition = {1: [1, 2, 4, 5, 6, 7]};
const genau: WordDefinition = {3: [7, 8, 9, 10, 11]};
const ab: WordDefinition = {4: [4, 5]};
const vor: WordDefinition = {4: [1, 2, 3]};
const haubi: WordDefinition = {4: [7, 8, 9, 10, 11]};
const fuef: WordDefinition = {1: [9, 10, 11]};
const zae: WordDefinition = {2: [9, 10, 11]};
const viertu: WordDefinition = {2: [1, 2, 3, 4, 5, 6]};
const zwaenzg: WordDefinition = {3: [1, 2, 3, 4, 5, 6]};

export const LayoutDeChGenau: Layout = {
    code: 'de_CH_genau',
    hours: {
        '0,12': {10: [1, 2, 3, 4, 5, 6]},
        '1,13': {5: [1, 2, 3]},
        '2,14': {5: [4, 5, 6, 7]},
        '3,15': {5: [9, 10, 11]},
        '4,16': {6: [1, 2, 3, 4, 5]},
        '5,17': {6: [6, 7, 8, 9]},
        '6,18': {7: [1, 2, 3, 4, 5, 6]},
        '7,19': {7: [7, 8, 9, 10, 11]},
        '8,20': {8: [1, 2, 3, 4, 5]},
        '9,21': {8: [6, 7, 8, 9]},
        '10,22': {9: [1, 2, 3, 4]},
        '11,23': {9: [8, 9, 10, 11]}
    },
    letters: [
        'ESKISCHAFÜF',
        'VIERTUBFZÄÄ',
        'ZWÄNZGGENAU',
        'VORABOHAUBI',
        'EISZWÖISDRÜ',
        'VIERIFÜFIQT',
        'SÄCHSISIBNI',
        'ACHTINÜNIEL',
        'ZÄNIERBEUFI',
        'ZWÖUFINAUHR'
    ],
    minutes: {
        '0': genau,
        '5,6,7,8,9': [fuef, ab],
        '10,11,12,13,14': [zae, ab],
        '15,16,17,18,19': [viertu, ab],
        '20,21,22,23,24': [zwaenzg, ab],
        '25,26,27,28,29': [fuef, vor, haubi],
        '30,31,32,33,34': haubi,
        '35,36,37,38,39': [fuef, ab, haubi],
        '40,41,42,43,44': [zwaenzg, vor],
        '45,46,47,48,49': [viertu, vor],
        '50,51,52,53,54': [zae, vor],
        '55,56,57,58,59': [fuef, vor]
    },
    permanent: esIsch,
    prettyName: 'Bärndütsch (genau)',
    version: 2
};

declare namespace Fritteli {
    interface Fritteli {
        uhr: Uhr;
    }

    interface Uhr {
        register: (layout: Layout) => void;
    }
}
declare namespace $ {
    const fritteli: Fritteli.Fritteli;
}

$.fritteli.uhr.register(LayoutDeChGenau);
